package org.processmining.models.animation.visualization;

import java.awt.Color;
import java.awt.Dimension;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.processmining.models.animation.Animation;

import com.fluxicon.slickerbox.components.RoundedPanel;

public abstract class AnimationMeter extends RoundedPanel implements AnimationListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5001723143698399584L;
	
	private Color bgColor = new Color(0, 0, 0);
	private Color labelColor = new Color(200, 200, 200, 80);
	private Color valueColor = new Color(200, 200, 200, 120);
	
	private Animation animation;
	
	private long updateInterval = 500;
	private long lastUpdate = 0;
	
	protected long modelTime;
	private RoundGauge gauge;
	private JLabel label;
	private JLabel value;
	
	protected AnimationMeter(Animation anim, String name) {
		super(15, 5, 0);
		Dimension size = new Dimension(170, 70);
		this.setMinimumSize(size);
		this.setMaximumSize(size);
		this.setPreferredSize(size);
		this.animation = anim;
		this.setBackground(bgColor);
		this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		this.gauge = new RoundGauge(50, 0);
		this.label = new JLabel(name);
		this.label.setOpaque(true);
		this.label.setBackground(bgColor);
		this.label.setFont(this.label.getFont().deriveFont(11f));
		this.label.setForeground(labelColor);
		this.label.setAlignmentX(JLabel.LEFT_ALIGNMENT);
		this.label.setHorizontalAlignment(JLabel.LEFT);
		this.label.setHorizontalTextPosition(JLabel.LEFT);
		this.value = new JLabel("");
		this.value.setOpaque(true);
		this.value.setBackground(bgColor);
		this.value.setFont(this.value.getFont().deriveFont(18f));
		this.value.setForeground(valueColor);
		this.value.setAlignmentX(JLabel.LEFT_ALIGNMENT);
		this.value.setHorizontalAlignment(JLabel.LEFT);
		this.value.setHorizontalTextPosition(JLabel.LEFT);
		JPanel labelPanel = new JPanel();
		labelPanel.setOpaque(true);
		labelPanel.setBackground(bgColor);
		labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.Y_AXIS));
		labelPanel.setBorder(BorderFactory.createEmptyBorder());
		labelPanel.add(this.label);
		labelPanel.add(Box.createVerticalStrut(4));
		labelPanel.add(this.value);
		labelPanel.add(Box.createVerticalGlue());
		this.add(this.gauge);
		this.add(Box.createHorizontalStrut(5));
		this.add(labelPanel);
	}

	/* (non-Javadoc)
	 * @see org.processmining.mining.fuzzymining.vis.anim.AnimationListener#started()
	 */
	public void started() {
		// ignore
	}

	/* (non-Javadoc)
	 * @see org.processmining.mining.fuzzymining.vis.anim.AnimationListener#stopped()
	 */
	public void stopped() {
		// ignore
	}

	/* (non-Javadoc)
	 * @see org.processmining.mining.fuzzymining.vis.anim.AnimationListener#updateModelTime(long)
	 */
	public void updateModelTime(long modelTime) {
		long current = System.currentTimeMillis();
		if((current - lastUpdate) > updateInterval) {
			lastUpdate = current;
			update(modelTime);
		}
	}
	
	protected Animation getAnimation() {
		return animation;
	}
	
	protected void setState(float percentage, String valueText) {
		this.gauge.setPercentage(percentage);
		this.value.setText(valueText);
	}
	
	protected abstract void update(long modelTime);
}
