package org.processmining.models.animation.visualization;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.processmining.models.animation.Animation;
import org.processmining.models.animation.visualization.painters.ColorSettings;

import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

public class AnimationPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -6958641930979684087L;

//	private Animation animation;
	private AnimationTimer timer;
	private AnimationCanvas canvas;

	public AnimationPanel(Animation animation) {

//		this.animation = animation;
		setupAnimation(animation);
	}

	public void setupAnimation(Animation animation) {
		double speedFactor = animation.getMeanBetweenTime() / 1000.0;
		
		timer = new AnimationTimer(animation.getStart(), animation.getEnd(),
				speedFactor, 20);
		canvas = new UnbufferedAnimationCanvas(animation,
				ColorSettings.SETTINGS_NIGHT);
		timer.addListener(canvas);
		
		ControlPanel controlPanel = new ControlPanel(this);
		
		AnimationSidebar sidebar = new AnimationSidebar(animation, timer);
		sidebar.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		sidebar.setLayout(new BoxLayout(sidebar, BoxLayout.Y_AXIS));
		
		JScrollPane scrollPane = new JScrollPane(canvas);
		scrollPane.getHorizontalScrollBar().setUI(
				new SlickerScrollBarUI(scrollPane.getHorizontalScrollBar()));
		scrollPane.getVerticalScrollBar().setUI(
				new SlickerScrollBarUI(scrollPane.getVerticalScrollBar()));
		scrollPane
				.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		scrollPane
				.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setBorder(BorderFactory.createEmptyBorder());
		scrollPane.setBackground(Color.BLACK);
		
		setBorder(BorderFactory.createEmptyBorder());
		setLayout(new BorderLayout());
		add(scrollPane, BorderLayout.CENTER);
		add(controlPanel, BorderLayout.SOUTH);
		add(sidebar, BorderLayout.WEST);
		revalidate();
	}

//	public XLog getLog() {
//		return animation.getLog();
//	}

//	public Animation getAnimation() {
//		return animation;
//	}

//	public DirectedGraph<? extends AbstractDirectedGraphNode, ? extends AbstractDirectedGraphEdge<?, ?>> getGraph() {
//		return animation.getGraph();
//	}

	public AnimationCanvas getCanvas() {
		return canvas;
	}

	public AnimationTimer getTimer() {
		return timer;
	}
}
