package org.processmining.models.animation.visualization;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.processmining.framework.util.ui.ExpandButton;
import org.processmining.models.animation.Animation;

import com.fluxicon.slickerbox.components.RoundedPanel;

public class AnimationSidebar extends JPanel implements AnimationListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = -8041446574539590035L;
	
	private Color colorLeft = new Color(40, 40, 40);
	private Color colorRight = new Color(20, 20, 20);
	private int width = 190;
	
	private boolean visible = true;
	protected Animation animation;
	protected AnimationTimer timer;
	
	private ActivityMeter activityMeter;
	private CompletionMeter completionMeter;
	private CaseAnimationPanel caseAnimationPanel;
	
	private ExpandButton expandButton;
	private JPanel expandPanel;
	private JPanel contentPanel;

	public AnimationSidebar(Animation animation, AnimationTimer timer) {
		contentPanel = new JPanel();
		contentPanel.setOpaque(false);
		contentPanel.setMinimumSize(new Dimension(width, 200));
		contentPanel.setMaximumSize(new Dimension(width, 2000));
		contentPanel.setPreferredSize(new Dimension(width, 1000));
		this.animation = animation;
		this.timer = timer;
		timer.addListener(this);
		this.setOpaque(true);
		contentPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		contentPanel.setLayout(new BoxLayout(contentPanel, BoxLayout.Y_AXIS));
		this.activityMeter = new ActivityMeter(animation);
		this.activityMeter.setAlignmentX(JPanel.CENTER_ALIGNMENT);
		this.completionMeter = new CompletionMeter(animation);
		this.completionMeter.setAlignmentX(JPanel.CENTER_ALIGNMENT);
		this.caseAnimationPanel = new CaseAnimationPanel(animation);
		RoundedPanel caseView = new RoundedPanel(15, 5, 0);
		caseView.setAlignmentX(JPanel.CENTER_ALIGNMENT);
		caseView.setMinimumSize(new Dimension(170, 100));
		caseView.setMaximumSize(new Dimension(170, 2000));
		caseView.setPreferredSize(new Dimension(170, 800));
		caseView.setBackground(new Color(0, 0, 0));
		caseView.setLayout(new BorderLayout());
		JLabel caseViewLabel = new JLabel("case progress");
		caseViewLabel.setFont(caseViewLabel.getFont().deriveFont(12f));
		caseViewLabel.setBackground(Color.BLACK);
		caseViewLabel.setForeground(new Color(60, 60, 60));
		caseView.add(caseViewLabel, BorderLayout.NORTH);
		caseView.add(this.caseAnimationPanel);
		contentPanel.add(this.activityMeter);
		contentPanel.add(this.completionMeter);
		contentPanel.add(Box.createHorizontalStrut(10));
		contentPanel.add(caseView);
		contentPanel.add(Box.createHorizontalStrut(5));
		expandButton = new ExpandButton();
		expandButton.setAlignmentX(ExpandButton.LEFT_ALIGNMENT);
		expandButton.setExpanded(true);
		expandButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(expandButton.isExpanded() == true) {
					// fold
					visible = false;
					expandButton.setExpanded(false);
					removeAll();
					add(expandPanel);
					add(Box.createVerticalGlue());
					revalidate();
					repaint();
				} else {
					// expand
					visible = true;
					expandButton.setExpanded(true);
					removeAll();
					add(expandPanel);
					add(contentPanel);
					revalidate();
					repaint();
				}
			}		
		});
		expandPanel = new JPanel();
		expandPanel.setMaximumSize(new Dimension(200, 10));
		expandPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
		expandPanel.setOpaque(false);
		expandPanel.setLayout(new BoxLayout(expandPanel, BoxLayout.X_AXIS));
		expandPanel.add(expandButton);
		expandPanel.add(Box.createHorizontalGlue());
		this.setBorder(BorderFactory.createEmptyBorder());
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.add(expandPanel);
		this.add(contentPanel);
	}

	/* (non-Javadoc)
	 * @see org.processmining.mining.fuzzymining.vis.anim.AnimationListener#started()
	 */
	public void started() {
		activityMeter.started();
		completionMeter.started();
		caseAnimationPanel.started();
	}

	/* (non-Javadoc)
	 * @see org.processmining.mining.fuzzymining.vis.anim.AnimationListener#stopped()
	 */
	public void stopped() {
		activityMeter.stopped();
		completionMeter.stopped();
		caseAnimationPanel.stopped();
	}

	/* (non-Javadoc)
	 * @see org.processmining.mining.fuzzymining.vis.anim.AnimationListener#updateModelTime(long)
	 */
	public void updateModelTime(long modelTime) {
		if(this.visible == true) {
			activityMeter.updateModelTime(modelTime);
			completionMeter.updateModelTime(modelTime);
			caseAnimationPanel.updateModelTime(modelTime);
		}
	}
	
	protected void paintComponent(Graphics g) {
		Graphics2D g2d = (Graphics2D)g.create();
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2d.setFont(g2d.getFont().deriveFont(9f));
		float width = this.getWidth();
		float height = this.getHeight();
		// fill background
		g2d.setColor(Color.BLACK);
		g2d.fill(new Rectangle2D.Float(0, 0, width + 1, height + 1));
		if(this.visible == true) {
			GradientPaint gradient = new GradientPaint(width - 4, 0, colorLeft, 
					width, 0, colorRight, false);
			g2d.setPaint(gradient);
			g2d.fill(new RoundRectangle2D.Float(-30, 0, width + 30, height, 30f, 30f));
		}
	}
}
