package org.processmining.models.animation.visualization;

import java.text.DecimalFormat;

import org.processmining.models.animation.Animation;
import org.processmining.models.animation.EdgeAnimation;
import org.processmining.models.animation.EdgeAnimationKeyframe;
import org.processmining.models.animation.NodeAnimation;

public class CompletionMeter extends AnimationMeter {

	/**
	 * 
	 */
	private static final long serialVersionUID = 4288344970698151428L;

	protected DecimalFormat format = new DecimalFormat("##0 %");

	/**
	 * @param anim
	 * @param name
	 */
	public CompletionMeter(Animation anim) {
		super(anim, "Completion");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.processmining.mining.fuzzymining.vis.ui.AnimationMeter#update(long)
	 */
	@Override
	protected void update(long modelTime) {
		float maxActivity = 0;
		float activity = 0;
		long endTime = getAnimation().getEnd();
		for (NodeAnimation anim : getAnimation().getNodeAnimations()) {
			maxActivity += anim.getKeyframe(endTime).getEventCount();
			activity += anim.getKeyframe(modelTime).getEventCount();
		}
		for (EdgeAnimation anim : getAnimation().getEdgeAnimations()) {
			maxActivity += anim.getKeyframe(endTime).getTraverseCount();
			EdgeAnimationKeyframe keyframe = anim.getKeyframe(modelTime);
			activity += keyframe.getTraverseCount();
			activity += keyframe.getTokenAnimations().size();
		}
		activity = activity / maxActivity;
		if (activity > 1f) {
			activity = 1f;
		}
		this.setState(activity, format.format(activity));
	}
}
