package org.processmining.models.animation.visualization;

import java.awt.Color;
import java.awt.Dimension;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.processmining.contexts.uitopia.hub.overlay.ProgressOverlayDialog.CancellationListener;

import com.fluxicon.slickerbox.components.RoundedPanel;
import com.fluxicon.slickerbox.components.SlickerProgressBar;

public class ProgressPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -5978617950787628576L;

	private static Color colorBgInner = new Color(200, 200, 200, 120);
	private SlickerProgressBar progress;
	private JLabel title;
	private JLabel label;

	public ProgressPanel(String aTitle, final CancellationListener cancellationListener) {
		
		RoundedPanel innerPanel = new RoundedPanel(20, 0, 0);
		innerPanel.setAlignmentX(JPanel.CENTER_ALIGNMENT);
		innerPanel.setBackground(colorBgInner);
		innerPanel.setMinimumSize(new Dimension(300, 120));
		innerPanel.setMaximumSize(new Dimension(600, 130));
		innerPanel.setPreferredSize(new Dimension(500, 130));
		innerPanel.setLayout(new BoxLayout(innerPanel, BoxLayout.Y_AXIS));
		innerPanel.setBorder(BorderFactory.createEmptyBorder(5, 20, 5, 20));

		progress = new SlickerProgressBar();
		progress.setOpaque(false);
		progress.setIndeterminate(true);
		
		title = new JLabel(aTitle);
		title.setAlignmentX(JLabel.LEFT_ALIGNMENT);
		title.setOpaque(false);
		title.setFont(title.getFont().deriveFont(16.0f));

		label = new JLabel("");
		label.setFont(label.getFont().deriveFont(12.0f));
		label.setAlignmentX(JLabel.LEFT_ALIGNMENT);
		label.setOpaque(false);

		JPanel buttonPanel = new JPanel();
		buttonPanel.setOpaque(false);
		buttonPanel.setBorder(BorderFactory.createEmptyBorder());
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(progress);
		buttonPanel.add(Box.createHorizontalStrut(10));

		innerPanel.add(Box.createVerticalGlue());
		innerPanel.add(alignLeft(title));
		innerPanel.add(Box.createVerticalStrut(10));
		innerPanel.add(buttonPanel);
		innerPanel.add(Box.createVerticalStrut(8));
		innerPanel.add(alignLeft(label));
		innerPanel.add(Box.createVerticalGlue());	
		
		add(Box.createHorizontalGlue());
		add(innerPanel);
		add(Box.createHorizontalGlue());
		this.setBackground(Color.BLACK);
		
	}
	
	protected JPanel alignLeft(JComponent component) {
		JPanel enclosure = new JPanel();
		enclosure.setOpaque(false);
		enclosure.setBorder(BorderFactory.createEmptyBorder());
		enclosure.setLayout(new BoxLayout(enclosure, BoxLayout.X_AXIS));
		enclosure.add(component);
		enclosure.add(Box.createHorizontalGlue());
		return enclosure;
	}

	protected JPanel alignRight(JComponent component) {
		JPanel enclosure = new JPanel();
		enclosure.setOpaque(false);
		enclosure.setBorder(BorderFactory.createEmptyBorder());
		enclosure.setLayout(new BoxLayout(enclosure, BoxLayout.X_AXIS));
		enclosure.add(Box.createHorizontalGlue());
		enclosure.add(component);
		return enclosure;
	}
	public void changeProgress(int p) {
		progress.setValue(p);
	}

	public void changeProgressBounds(int lowBo, int upBo) {
		progress.setMinimum(lowBo);
		progress.setMaximum(upBo);
		progress.setIndeterminate(upBo <= lowBo);
	}

	public void changeProgressCaption(String newCaption) {
		label.setText(newCaption);
		revalidate();
	}

	public void changeProgressIndeterminate(boolean indeterminate) {
		progress.setIndeterminate(indeterminate);
	}

	public int getMaximum() {
		return progress.getMaximum();
	}

	public void setIndeterminate(boolean b) {
		progress.setIndeterminate(b);

	}
	public void setMaximum(int max){
		progress.setMaximum(max);
	}
	
	public void setMinimum(int min){
		progress.setMinimum(min);
	}
	
	public int getValue(){
		return progress.getValue();
	}
}

