package org.processmining.models.animation.visualization;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import org.processmining.models.animation.Animation;
import org.processmining.models.animation.visualization.painters.ColorSettings;

public class UnbufferedAnimationCanvas extends AnimationCanvas {

	/**
	 * 
	 */
	private static final long serialVersionUID = -6460534086899448661L;
	
	private long lastPaintTime = System.currentTimeMillis();

	public UnbufferedAnimationCanvas(Animation animation,
			ColorSettings settings) {
		super(animation, settings);
		this.backBuffer = null;
		this.buffer = null;
		this.setOpaque(true);
		this.setDoubleBuffered(true);
	}

	public void setZoom(float zoom) {
		this.zoom = zoom;
		// leftBorder = boxHeight * animation.getParser().getBoundsHeight() *
		// SCALE_FACTOR * zoom;
		// double jgraphWidth = animation.getParser().getBoundsWidth();
		// double jgraphHeight = animation.getParser().getBoundsHeight();
		leftBorder = animation.getLayout().getBoundsHeight() * zoom * 0.35;
		topBorder = leftBorder / 3;
		bottomBorder = topBorder;
		rightBorder = leftBorder * 3;
		// this.width = animation.getParser().getBoundsWidth() * SCALE_FACTOR *
		// zoom;
		// this.height = animation.getParser().getBoundsHeight() * SCALE_FACTOR
		// * zoom;
		this.width = animation.getLayout().getBoundsWidth() * zoom;
		this.height = animation.getLayout().getBoundsHeight() * zoom;
		int imgWidth = (int) (width + leftBorder + rightBorder);
		int imgHeight = (int) (height + topBorder + bottomBorder);
		Dimension size = new Dimension(imgWidth, imgHeight);
		this.setMinimumSize(size);
		this.setPreferredSize(size);
		revalidate();
		if (this.isDisplayable()) {
			repaint();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.processmining.mining.fuzzymining.anim.model.AnimationListener#
	 * updateModelTime(long)
	 */
	public void updateModelTime(long modelTime) {
		this.animationTime = modelTime;
		repaint();
	}

	protected long getRenderTime() {
		long current = System.currentTimeMillis();
		long renderTime = current - this.lastPaintTime;
		lastPaintTime = current;
		return renderTime;
	}

	protected double getFramesPerSecond() {
		return 1000.0 / getRenderTime();
	}

	protected void paintComponent(Graphics g) {
		long modelTime = this.animationTime;
		Graphics2D g2d = (Graphics2D) g;
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		int x = 0;
		int y = 0;
		int actualWidth = this.getWidth();
		int actualHeight = this.getHeight();
		int canvasWidth = (int) (this.width + leftBorder + rightBorder);
		int canvasHeight = (int) (this.height + topBorder + bottomBorder);
		if (actualWidth > canvasWidth) {
			x = (actualWidth - canvasWidth) / 2;
		}
		if (actualHeight > canvasHeight) {
			y = (actualHeight - canvasHeight) / 2;
		}
		g2d.setColor(colorSettings.getCanvasBackground());
		g.fillRect(0, 0, actualWidth + 1, actualHeight + 1);
		// translate origin
		g2d.translate(leftBorder + x, topBorder + y);
		// paint to back buffer
		// paint layer 0: arc base trajectory
		Graphics2D g2dCopy = (Graphics2D) g2d.create();
		this.edgeBackgroundPainter.paint(modelTime, g2dCopy);
		// paint layer 1: token trace trajectories
		this.edgeTrajectoryPainter.paint(modelTime, g2dCopy);
		// paint layer 2: task bases
		this.nodePainter.paint(modelTime, g2dCopy);
		// test: token rings
		this.tokenRingsPainter.paint(modelTime, g2dCopy);
		// paint layer 3: tokens
		this.tokenPainter.paint(modelTime, g2dCopy);
		// paint layer 4: token labels
		this.tokenLabelPainter.paint(modelTime, g2dCopy);
		// paint layer 5: task badges
		this.nodeBadgesPainter.paint(modelTime, g2dCopy);
	}

}

