package org.processmining.models.animation.visualization.painters;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.GeneralPath;
import java.util.List;

import org.processmining.models.animation.AnimationCurve;
import org.processmining.models.animation.EdgeAnimation;
import org.processmining.models.animation.EdgeAnimationKeyframe;
import org.processmining.models.animation.visualization.AnimationCanvas;

public class EdgeBackgroundPainter extends Painter {

	public EdgeBackgroundPainter(AnimationCanvas canvas) {
		super(canvas);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.processmining.mining.fuzzymining.vis.paint.Painter#paint(long,
	 * java.awt.Graphics2D)
	 */
	@Override
	public void paint(long modelTime, Graphics2D g2d) {
		EdgeAnimationKeyframe keyframe = null;
		AnimationCurve curve = null;
		double x[], y[];
		long fadeTime = canvas.getAnimation().getFadeTime();
		long idleTime;
		int traverseCount;
		int bestQuarterTraverseCount = canvas.getAnimation().getBestQuarterTraverseCount();
		float percentage;
		List<EdgeAnimation> animations = canvas.getAnimation().getEdgeAnimations();
		for (EdgeAnimation anim : animations) {
			keyframe = anim.getKeyframe(modelTime);
			// derive stroke width from traverse count and set
			traverseCount = keyframe.getTraverseCount();
			percentage = (float) traverseCount / (float) bestQuarterTraverseCount;
			if (percentage > 1.0f) {
				percentage = 1.0f;
			}
			g2d.setStroke(new BasicStroke((float) canvas.getLineWidth(percentage), BasicStroke.CAP_ROUND,
					BasicStroke.JOIN_ROUND));
			// derive stroke color from idle time and set
			idleTime = modelTime - keyframe.getLastCompleted();
			Color terminalColor;
			if (idleTime > fadeTime) {
				terminalColor = canvas.getColorSettings().getPassiveEdgeBackground();
			} else {
				percentage = (float) idleTime / (float) fadeTime;
				terminalColor = ColorSettings.mix(canvas.getColorSettings().getActiveEdgeBackground(), canvas
						.getColorSettings().getPassiveEdgeBackground(), percentage);
			}
			curve = anim.getCurve();

			x = curve.getPointsX();
			y = curve.getPointsY();
			GeneralPath path = new GeneralPath();
			double firstX = canvas.translateX(x[0]);
			double firstY = canvas.translateY(y[0]);
			path.moveTo((float) firstX, (float) firstY);
			for (int i = 1; i < x.length; i++) {
				double nextPointX = canvas.translateX(x[i]);
				double nextPointY = canvas.translateY(y[i]);
				path.lineTo((float) nextPointX, (float) nextPointY);
			}
			double lastPointX = canvas.translateX(x[x.length - 1]);
			double lastPointY = canvas.translateY(y[y.length - 1]);
			path.moveTo((float) lastPointX, (float) lastPointY);

			g2d.setColor(terminalColor);
			g2d.draw(path);
		}
	}

}
