package org.processmining.models.animation.visualization.painters;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.util.List;

import org.processmining.models.animation.NodeAnimation;
import org.processmining.models.animation.NodeAnimationKeyframe;
import org.processmining.models.animation.visualization.AnimationCanvas;

public class NodePainter extends Painter {

	public NodePainter(AnimationCanvas canvas) {
		super(canvas);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.processmining.mining.fuzzymining.vis.paint.Painter#paint(long,
	 * java.awt.Graphics2D)
	 */
	@Override
	public void paint(long modelTime, Graphics2D g2d) {
		BasicStroke stroke = new BasicStroke((float) canvas.getBorderWidth(), BasicStroke.CAP_ROUND,
				BasicStroke.JOIN_ROUND);
		g2d.setStroke(stroke);
		float fontSize = canvas.getFontSize();
		g2d.setFont(g2d.getFont().deriveFont(fontSize));
		long fadeTime = canvas.getAnimation().getFadeTime();
		List<NodeAnimation> nodeAnimations = canvas.getAnimation().getNodeAnimations();
		NodeAnimationKeyframe keyframe = null;
		for (NodeAnimation nodeAnimation : nodeAnimations) {
			keyframe = nodeAnimation.getKeyframe(modelTime);
			long idleTime = modelTime - keyframe.getTime();
			Color bgColor, textColor, borderColor;
			if (idleTime >= fadeTime) {
				bgColor = canvas.getColorSettings().getPassiveStateBackground();
				textColor = canvas.getColorSettings().getPassiveStateForeground();
				borderColor = canvas.getColorSettings().getPassiveStateBorder();
			} else {
				float percentage = (float) idleTime / (float) fadeTime;
				bgColor = ColorSettings.mix(canvas.getColorSettings().getActiveStateBackground(), canvas
						.getColorSettings().getPassiveStateBackground(), percentage);
				textColor = ColorSettings.mix(canvas.getColorSettings().getActiveStateForeground(), canvas
						.getColorSettings().getPassiveStateForeground(), percentage);
				borderColor = ColorSettings.mix(canvas.getColorSettings().getActiveStateBorder(), canvas
						.getColorSettings().getPassiveStateBorder(), percentage);
			}
			double x = canvas.translateX(nodeAnimation.getPositionX());
			double y = canvas.translateY(nodeAnimation.getPositionY());
			double width = canvas.translateX(nodeAnimation.getWidth());
			double height = canvas.translateY(nodeAnimation.getHeight());
			// paint background
			GradientPaint gradient = new GradientPaint((float) x, (float) y, bgColor, (float) x, (float) (y + height),
					ColorSettings.darken(bgColor, 0.8f), false);
			g2d.setPaint(gradient);
			canvas.getAnimation().paintNodeBackground(nodeAnimation.getNode(), g2d, x, y, width, height);
			// paint border
			gradient = new GradientPaint((float) x, (float) y, borderColor, (float) x, (float) (y + height),
					ColorSettings.fade(borderColor, 0.7f), false);
			g2d.setPaint(gradient);
			canvas.getAnimation().paintNodeBorder(nodeAnimation.getNode(), g2d, x, y, width, height);
			// paint text
			g2d.setColor(textColor);
			canvas.getAnimation().paintNodeText(nodeAnimation.getNode(), g2d, x, y, width, height);
		}
	}

}
