package org.processmining.models.animation.visualization.painters;

import java.awt.Graphics2D;
import java.awt.geom.Ellipse2D;
import java.util.List;

import org.processmining.models.animation.AnimationCurve;
import org.processmining.models.animation.EdgeAnimation;
import org.processmining.models.animation.EdgeAnimationKeyframe;
import org.processmining.models.animation.TokenAnimation;
import org.processmining.models.animation.visualization.AnimationCanvas;

public class TokenPainter extends Painter {

	public TokenPainter(AnimationCanvas canvas) {
		super(canvas);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.processmining.mining.fuzzymining.vis.paint.Painter#paint(long,
	 * java.awt.Graphics2D)
	 */
	@Override
	public void paint(long modelTime, Graphics2D g2d) {
		g2d.setColor(canvas.getColorSettings().getTokensBackground());
		double radius = canvas.translateY(canvas.getBoxHeight()) * 0.25f;
		double diameter = 2 * radius;
		EdgeAnimationKeyframe keyframe = null;
		AnimationCurve curve = null;
		List<EdgeAnimation> animations = canvas.getAnimation().getEdgeAnimations();
		for (EdgeAnimation anim : animations) {
			keyframe = anim.getKeyframe(modelTime);
			curve = anim.getCurve();
			// derive stroke color from idle time and set
			for (TokenAnimation tokenAnim : keyframe.getTokenAnimations()) {
				double[] pos = curve.getRelativePoint(tokenAnim.completedPercentage(modelTime));
				double x = canvas.translateX(pos[0]);
				double y = canvas.translateY(pos[1]);
				Ellipse2D.Double circle = new Ellipse2D.Double(x - radius, y - radius, diameter, diameter);
				g2d.fill(circle);
			}
		}
	}
}
