package org.processmining.models.animation.visualization.painters;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Ellipse2D;
import java.util.List;

import org.processmining.models.animation.AnimationCurve;
import org.processmining.models.animation.EdgeAnimation;
import org.processmining.models.animation.EdgeAnimationKeyframe;
import org.processmining.models.animation.TokenAnimation;
import org.processmining.models.animation.visualization.AnimationCanvas;

public class TokenRingsPainter extends Painter {

	public TokenRingsPainter(AnimationCanvas canvas) {
		super(canvas);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.processmining.mining.fuzzymining.vis.paint.Painter#paint(long,
	 * java.awt.Graphics2D)
	 */
	@Override
	public void paint(long modelTime, Graphics2D g2d) {
		int cycle = (int) (System.currentTimeMillis() % 3000);
		if (cycle < 2300) {
			return; // blank frame
		}
		double percentage = (cycle - 2300) / 700f;
		double boxHeight = canvas.translateY(canvas.getBoxHeight());
		double radius = boxHeight * 1.5f * percentage;
		Color color = ColorSettings.setAlpha(canvas.getColorSettings().getTokenRing(),
				(float) (0.3f * (1f - percentage)));
		g2d.setColor(color);
		BasicStroke stroke = new BasicStroke((float) ((1f - percentage) * 0.2f * boxHeight), BasicStroke.CAP_ROUND,
				BasicStroke.JOIN_ROUND);
		g2d.setStroke(stroke);
		double diameter = 2 * radius;
		EdgeAnimationKeyframe keyframe = null;
		AnimationCurve curve = null;
		List<EdgeAnimation> animations = canvas.getAnimation().getEdgeAnimations();
		for (EdgeAnimation anim : animations) {
			keyframe = anim.getKeyframe(modelTime);
			curve = anim.getCurve();
			for (TokenAnimation tokenAnim : keyframe.getTokenAnimations()) {
				double[] pos = curve.getRelativePoint(tokenAnim.completedPercentage(modelTime));
				double x = canvas.translateX(pos[0]);
				double y = canvas.translateY(pos[1]);
				g2d.draw(new Ellipse2D.Double(x - radius, y - radius, diameter, diameter));
			}
		}
	}

}

